Package: SChangeBlock
Title: Spatial Structural Change Detection by an Analysis of
        Variability Between Blocks of Observations
Version: 0.1.1
Authors@R: 
    c(person("Sheila", "Goerz", email = "sheila.goerz@tu-dortmund.de", role = c("aut", "cre")), person("Roland", "Fried", email = "msnat@statistik.tu-dortmund.de", role = c("ctb", "ths")))
Description: Provides methods to detect structural changes in time series or random fields (spatial data). Focus is on the detection of abrupt changes or trends in independent data, but the package also provides a function to de-correlate data with dependence. The functions are based on the test suggested in Schmidt (2024) <DOI:10.3150/23-BEJ1686> and the work in Görz and Fried (2025) <DOI:10.48550/arXiv.2512.11599>.
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.2
Imports: Rcpp, ggplot2, nortest, expm, robcp, methods
Depends: R (>= 3.3.1)
LinkingTo: Rcpp
Author: Sheila Goerz [aut, cre],
  Roland Fried [ctb, ths]
Maintainer: Sheila Goerz <sheila.goerz@tu-dortmund.de>
Suggests: testthat (>= 3.0.0)
Config/testthat/edition: 3
Date: 2026-05-26
NeedsCompilation: yes
Packaged: 2026-05-26 14:14:56 UTC; goerz
Repository: CRAN
Date/Publication: 2026-05-26 14:40:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-05-26 16:30:46 UTC; unix
Archs: SChangeBlock.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(plot,RandomField)
export(GMD)
export(Mu)
export(autocov)
export(bandwidth)
export(block_pValue)
export(block_stat)
export(block_test)
export(changeRegion)
export(crit.grubbs)
export(decorr)
export(genField)
export(genTheta)
export(grubbs)
export(kurtosis)
export(rmix)
export(sOpt)
export(sSizes)
export(skewness)
import(Rcpp)
import(stats)
importFrom(Rcpp,evalCpp)
importFrom(expm,sqrtm)
importFrom(ggplot2,aes)
importFrom(ggplot2,geom_raster)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,guide_colorbar)
importFrom(ggplot2,guides)
importFrom(ggplot2,scale_fill_gradientn)
importFrom(grDevices,rgb)
importFrom(methods,is)
importFrom(nortest,ad.test)
importFrom(nortest,lillie.test)
importFrom(robcp,modifChol)
useDynLib(SChangeBlock)

Provides methods to detect structural changes in time series or random fields (spatial data). Focus is on the detection of abrupt changes or trends in independent data, but the package also provides a function to de-correlate data with dependence. The functions are based on the test suggested in Schmidt (2024) ([doi:10.3150/23-BEJ1686](https://doi.org/10.3150%2F23-BEJ1686)) and the work in Görz and Fried (2025) ([doi:10.48550/arXiv.2512.11599](https://doi.org/10.48550%2FarXiv.2512.11599)).

Provides methods to detect structural changes in time series or random fields (spatial data). Focus is on the detection of abrupt changes or trends in independent data, but the package also provides a function to de-correlate data with dependence. The functions are based on the test suggested in Schmidt (2024) <DOI:10.3150/23-BEJ1686> and the work in Görz and Fried (2025+) ...

SP <- function(X, lag = 1)
{
  n <- nrow(X)
  m <- ncol(X)
  
  rho1 <- abs(mean(sapply(1:m, function(i) cor(X[1:(n-lag), i], X[-c(1:lag), i], method = "spearman"))))
  rho2 <- abs(mean(sapply(1:n, function(i) cor(X[i, 1:(m-lag)], X[i, -c(1:lag)], method = "spearman"))))
  
  c(rho1 = rho1, rho2 = rho2)
}


#'Bandwidth estimation 
#'
#'Calculate MSE-optimal bandwidths according to Andrews (1991).
#'
#'@param X numeric vector or matrix.
#'@param p1,p2 exponents for sample size n resp. estimated dependency, between 0 and 1.
#'@param lag lag to which the autocorrelations are to be estimated. Integer > 0 but smaller than the length resp. number of rows and columns of X.
#'
#'@details Bandwidth \eqn{\boldsymbol{b}^{(n,m)} = (b_1^{(n)}, b_2^{(m)})} is estimated via
#'         \deqn{b_i^{(k)} = \min\left(k-1, \max\left(1, k^{0.3} \left(\frac{2\rho_i}{1 - \rho_i^2}\right)^{0.3}\right)\right),} 
#'         where \eqn{\rho_1} and \eqn{\rho_2} are the mean row- resp. column-wise Spearman autocorrelations to lag 1.
#'
#'@returns A numeric vector containing one or two elements, depending on if a vector or matrix is supplied.
#'         In case of a matrix: the first value is the bandwidth for the row-wise 
#'         and the second one for the column-wise estimation.
#'         
#'@references Andrews, D. W. (1991). “Heteroskedasticity and autocorrelation consistent covariance 
#'            matrix estimation”. In: Econometrica: Journal of the Econometric Society, pp. 817–858.
#'
#'@examples 
#'X1 <- genField(c(50, 50), Theta = genTheta(1, 0.4))
#'bandwidth(X1, 0.3, 0.3)
#'
#'Theta <- matrix(c(0.08, 0.1, 0.08, 0.8, 1, 0.8, 0.08, 0.1, 0.08), ncol = 3)
#'X2 <- genField(c(50, 50), Theta = Theta)
#'bandwidth(X2, 1/3, 2/3)
#'
#' @export
bandwidth <- function(X, p1 = 0.3, p2 = 0.3, lag = 1)
{
  if(is.matrix(X))
  {
    n <- nrow(X)
    m <- ncol(X)
    
    # ln <- round(n^s)
    # lm <- round(m^s)

    if(lag < 1 | lag >= n | lag >= m) stop("Wrong lag supplied!")
    
    # Res1 <- numeric(floor(n / ln) * floor(m / lm))
    # Res2 <- Res1
    # 
    # index <- 1
    # 
    # sapply(seq(1, n - ln + 1, ln), function(i)
    # {
    #   t(sapply(seq(1, m - lm + 1, lm), function(j)
    #   {
    #     Res1[index] <<- mean(sapply(i:(i+ln-1), function(l) cor(X[l, j:(j+lm-2)], X[l, (j+1):(j+lm-1)], method = "spearman")))
    #     Res2[index] <<- mean(sapply(j:(j+lm-1), function(k) cor(X[i:(i+ln-2), k], X[(i+1):(i+ln-1), k], method = "spearman")))
    #     
    #     index <<- index + 1
    #   }))
    # })
    # 
    # rho1 <- abs(median(Res1))
    # rho2 <- abs(median(Res2))
    
    rho <- SP(X, lag = lag)

    param1 <- min(max(round(m^(p1) * ((2 * rho[1]) / (1 - rho[1]^2))^(p2)), 0), m-1)
    param2 <- min(max(round(n^(p1) * ((2 * rho[2]) / (1 - rho[2]^2))^(p2)), 0), n-1)
    
    if(is.na(param1)) param1 <- 1
    if(is.na(param2)) param2 <- 1
    
    return(c(param1, param2))
  } else
  {
    n <- length(X)
    if(lag < 1 | lag >= n) stop("Wrong lag supplied!")
    
    rho <- abs(cor(X[1:(n-lag)], X[(1+lag):n], method = "spearman"))
    return(min(max(round(n^p1 * (rho / (1 - rho))^p2), 1), n-1))
  }
}


#' De-correlation
#' 
#' De-correlates a random field or time series, so that the resulting values can be treated as independent.
#' 
#' @param X Random Field, numeric matrix, or time series
#' @param lags numeric vector containing two integer values: the bandwidths for the row- reps. column-wise autocovariance estimation.
#'             (Up to which lag should the autocovariances be estimated?) Lags must be smaller than the dimensions of X.
#' @param method 1L: square root of the matrix via [robcp::modifChol()], inversion via [solve()] \cr
#'               2L: square root and inversion via singular value decomposition \cr
#'               3L: square root via [expm::sqrtm()], inversion via [solve()] \cr
#'               Otherwise, a custom function for the square root and the inversion can be supplied via \code{method}. This function must take exactly one argument, which is the matrix \code{X}.
#' @param separable if the autocovariance function is (assumed to be) separable in the two directions of X, 
#'                  those two autocovariances can be estimated separately and then combined (after square root and inversion) as a Kronecker product.
#' @param M numeric vector containing two integer values, only needed for \code{type = 1}, see [autocov()].
#' @param type 0: ordinary autocovariance estimation, 1: difference-based autocovariance estimation. See [autocov()].
#'                  
#' @details
#' The contents of \code{X} are ordered into a vector \eqn{x} column-wise. The autocovariance matrix \eqn{\Sigma} of \eqn{x} is estimated by [autocov()]. 
#' \eqn{\Sigma} is taken the square root of and being inverted using the functions specified in \code{method}. Then
#' \deqn{y = \Sigma^{-\frac{1}{2}} (x - \bar{x}).}
#' Then \eqn{y} is ordered back into a matrix \eqn{Y} with the same dimension as \eqn{X}.
#' 
#'                  
#' @returns De-correlated random field or time series; same data type and size as input \code{X}.
#'
#' @examples
#' x <- arima.sim(list(ar = 0.4), 200)
#' y <- decorr(x, 3)
#' \donttest{
#' oldpar <- par(mfrow = c(2, 2))
#' acf(x)
#' pacf(x)
#' acf(y)
#' pacf(y)
#' par(oldpar) }
#' 
#' X <- genField(c(20, 20), Theta = genTheta(1, 0.4))
#' Y <- decorr(X, c(2, 2))
#' 
#' \donttest{
#' # custom function:
#' require(robcp)
#' Y2 <- decorr(X, c(2, 2), method = function(x) modifChol(solve(x))) }
#' 
#'
#' @importFrom robcp modifChol
#' @importFrom expm sqrtm
#' @export
decorr <- function(X, lags, method = 1L, separable = FALSE, M = 1, type = 0)
{
  # if(is.ts(X)) 
  # {
  #   tspX <- tsp(X)
  #   clsX <- NULL
  # } else
  # { 
  #   tspX <- NULL
  #   clsX <- class(X)
  # }
  
  attrX <- attributes(X)
  
  if(is.vector(X) | is.null(dim(X)))
  {
    lags[2] <- 0
    M[2] <- 0
    X <- as.matrix(X)
  } else 
  {
    if(length(lags) == 1) lags[2] <- lags[1]
    if(length(M) == 1) M[2] <- M[1]
  }
  
  x <- as.vector(X)
  X <- as.matrix(X)

  if(separable)
  {
    A1 <- autocov(X, c(1, lags[2]), direction = 1, M = M, type = type)
    A2 <- autocov(X, c(lags[1], 1), direction = 2, M = M, type = type)
    
    A.invsqrt <- kronecker(invsqrt(A1, method), invsqrt(A2, method))
  } else 
  {
    A <- autocov(X, lags, M = M, type = type)
    A.invsqrt <- invsqrt(A, method)
  }

  y <- t(A.invsqrt) %*% (x - mean(x))
  if(ncol(X) > 1) Y <- matrix(y, ncol = ncol(X)) else Y <- as.vector(y)
  
  # if(!is.null(tspX)) 
  # {
  #   Y <- ts(Y)
  #   tsp(Y) <- tspX
  # } else
  # {
  #   class(Y) <- clsX
  # }
  
  attributes(Y) <- attrX
  
  return(Y)
}

invsqrt <- function(M, method = 1L)
{
  if(is.function(method))
  {
    return(method(M))
  } else if(method == 1L)
  {
    Mchol <- tryCatch(chol(M), error = function(e) e)
    if("error" %in% class(Mchol))
    {
      Mchol <- modifChol(M)
    }
    
    res <- solve(Mchol)
    
    return(res)
  } else if(method == 2L)
  {
    res <- svd(M)
    return(res$u %*% diag(1 / sqrt(res$d)) %*% t(res$v))
  } else if(method == 3L)
  {
    Mchol <- tryCatch(chol(M), error = function(e) e)
    if("error" %in% class(Mchol))
    {
      Mchol <- sqrtm(M)
    }
    return(solve(Mchol))
  } else 
  {
    stop("Wrong method")
  }
}

test_that("bandwidth estimation returns the correct format", {
  x <- arima.sim(list(ar = 0.5), 100)
  X <- genField(c(50, 50), Theta = genTheta(10, 0.4))

  expect_equal(length(bandwidth(x, 0.3, 0.3)), 1)
  expect_equal(length(bandwidth(X, 0.3, 0.3)), 2)
})


test_that("de-correlation is working correctly", {
  X <- genField(c(20, 20), Theta = genTheta(10, 0.3))
  Y <- decorr(X, lags = bandwidth(X, 0.3, 0.3))
  
  expect_equal(class(X), class(Y))
  
  x <- arima.sim(model = list(ar = 0.5), 100)
  y <- decorr(x, bandwidth(x, 0.3, 0.3))
  
  expect_equal(class(x), class(y))
  
  ## check if arima parameter is significant (to 10% level)
  arima.x <- arima(x, c(1, 0, 0))
  arima.y <- arima(y, c(1, 0, 0))
  
  expect_lt((1 - pnorm(abs(arima.x$coef[1]) / sqrt(arima.x$var.coef[1])))*2, 0.1)
  expect_gt((1 - pnorm(abs(arima.y$coef[1]) / sqrt(arima.y$var.coef[1])))*2, 0.1)
})

#' Mixture distribution
#' 
#' Generates a random sample from a mixture normal distribution.
#'
#' @param n sample size, numeric.
#' @param q probability that observation is drawn from the contamination distribution, numeric.
#' @param h mean of the contamination distribution, numeric.
#' @param sigma standard deviation of the contamination distribution, numeric.
#'
#' @details The resulting sample is drawn from the distribution
#' \deqn{(1 - q)\mathcal{N}(0, 1)\; + \; q \mathcal{N}(h, \sigma^2).}
#'
#' @return Numeric vector of length n containing the random sample.
#'
#' @examples
#' # random sample with 0.01 chance of contamination distribution with mean 10
#' rmix(100)
#' 
#' # random sample with 0.01 chance of contamination distribution with standard deviation 10
#' # IMPORTANT: h needs to be set to 0!
#' rmix(100, h = 0, sigma = 1)
#' 
#' @export
rmix <- function(n, q = 0.01, h = 10, sigma = 1)
{
  x <- rnorm(n)
  index <- sample(n, rbinom(1, n, q))
  x[index] <- sigma * x[index] + h
  return(x)
}

## usethis namespace: start
#' @import Rcpp 
#' @import stats
#' @importFrom Rcpp evalCpp
#' @importFrom grDevices rgb
#' @importFrom nortest lillie.test
#' @importFrom nortest ad.test
#' @useDynLib SChangeBlock
## usethis namespace: end
NULL

#' @keywords internal
#' 
#' @description
#' Provides methods to detect structural changes in time series or random fields (spatial data). Focus is on the detection of abrupt changes or trends in independent data, but the package also provides a function to de-correlate data with dependence. The functions are based on the test suggested in Schmidt (2024) <doi:10.3150/23-BEJ1686> and the work in Görz and Fried (2025+).
#' 
#' 
"_PACKAGE"

#' Generate random fields
#'
#'@description
#'Generate random fields on a regular grid. Dependency can be modeled to some extend.
#'
#'
#' @param n dimensions of the requested random field. Numeric vector.
#' @param distr function specifying the error distribution.
#' @param type change region type (integer or character). See \code{\link{changeRegion}}.
#' @param H height of the location shift (numeric value).
#' @param Theta matrix specifying the dependency between observations of a 2-dim random field. 
#'            Has to be a 2-dim. matrix where there is an odd number of entries in both dimensions. 
#'            Explanations are given in [genTheta].
#' @param q dependency parameter for the model order of the MA field (integer > 0). 
#' @param param dependency parameters for the model parameters of the MA field (numeric vector.) 
#'              Both \code{q} and \code{param} are ignored if \code{Theta} is supplied.
#'              A dependency matrix is generated using \code{\link{genTheta}}.
#' @param ... additional arguments for the generation of \code{\link{changeRegion}}.
#'
#' @details
#' Set \code{dim(X)} \eqn{= (n, m)} and \eqn{N = nm}. The dependent random field follows the equation:
#' \deqn{Y_{ij} = \sum_{k = -q}^q \sum_{l = -q}^q \theta_{kl} \varepsilon_{i+k, j+l} + \frac{H}{\sqrt{N}} \mathbb{1}_{\{(i, j) \in \mathbb{A}\}}}
#' where \eqn{\mathbb{A} = \{(i, j) \in \{1, ..., n\} \times \{1, ..., m\} | \text{ change region at } (i, j)\}}.
#' Only for \code{type = 3L} the term \eqn{\mathbb{1}_{\{(i, j) \in \mathbb{A}\}}} is replaced by \eqn{\frac{H}{\sqrt{N}} \frac{j}{m - 1}}.
#' 
#'
#' @returns
#' The function returns a matrix of dimension \code{n} that has the \code{\link{class}} "RandomField".
#'
#' @examples
#' genField(c(50, 50))
#' genField(c(50, 50), type = 3, Theta = genTheta(1, 0.2))
#' 
#' @seealso [changeRegion], [genTheta]
#'
#' @export
genField <- function(n, distr = rnorm, type = 0L, H = 100, 
                     Theta = NULL, q = NULL, param = NULL, ...) 
{
  ## is there supposed to be dependency? If yes, the error matrix has to be
  ## larger than the requested field:
  if(!is.null(Theta))
  {
    pn <- dim(Theta)
    if(any(pn %% 2 == 0)) stop("Dimensions of Theta are incorrect!")
    pn2 <- n + pn - 1
  } else if(!is.null(q) && !is.null(param))
  {
    pn2 <- n + 2 * q
  } else
  {
    pn2 <- n
  }
  
  # error matrix
  X <- matrix(distr(prod(pn2)), nrow = pn2[1])
  
  if(!is.null(Theta) || !(is.null(q) || is.null(param))) 
  {
    # multiply error matrix X with dependency matrix Theta:
    X <- dependency(X, Theta, q, param)
  } 
  
  ## add change region:
  if(type != 3L && type != 0L)
  {
    alt <- numeric(prod(n))
    altIndex <- changeRegion(n, type = type, ...)
    alt[altIndex] <- H / prod(n)^(0.5)# / (2 * length(altIndex))
    X <- X + alt
  } else if(type == 3L)
  {
    X <- X + changeRegion(n, type = 3L) * H
  }
  
  class(X) <- "RandomField"
  return(X)
}


#' Change Region
#' 
#' Generates the indices for different types of change regions.
#'
#' @param n dimensions of the random field. Numeric vector.
#' @param s parameter for the size of the blocks, 0.5 < s < 1, block length \eqn{l_n = [n^s]}.
#' @param type change region type (integer or character). See "Details".
#' @param middle,delta,distFun parameters for type 4L. See "Details".
#' 
#' @return 
#' \item{Types 1a, 1b, 1c, 2, 4, 5}{A vector of indices.}
#' \item{Type 3}{A numeric (n x n) matrix containing the heights of the shifts at the corresponding locations.}
#' 
#' @details
#' Change region types: 
#' * 1L or "1a": exactly one block is shifted
#' * "1b": size of the shift region is one block, but the shift region lies in two blocks
#' * "1c": size of the shift region is one block, but the shift region lies in four blocks
#' * 2L: exactly half of the data is shifted
#' * 3L: there is a steady increase from left to right
#' * 4L: shift is at \eqn{\{X_{ij} | (i \leq \frac{n}{2} \wedge \frac{m}{4} < j \leq \frac{m}{2}) \vee (\frac{n}{4} < i \leq \frac{n}{2} \wedge j \leq \frac{m}{4})\}}
#' * 5L: for demonstration purposes: the field is divided into 10 "columns". Every other column is shifted
#' * 6L: shift is at \eqn{\{X_{ij} | i \leq j \leq m\}} (upper triangle)
#' * 7L: a "circle" including everything within \code{distFun} \code{delta} from \code{middle} is shifted
#' 
#' @seealso [genField]
#' 
#' @examples
#' changeRegion(c(50, 50), 0.6, "1a")
#' changeRegion(c(50, 50), type = 2)
#' changeRegion(c(50, 50), type = 3L)
#' changeRegion(c(50, 50), type = 7L, middle = c(10, 10))
#' 
#' @export
changeRegion <- function(n, s, type = 1L, middle, delta = 0.15, distFun = dist)
{
  if(missing(s)) s <- sOpt(n, 0.6)
  ln <- round(n^s)

  if(is.na(n[2]))
  {
    n[2] <- 1
    ln[2] <- 1
  }

  alt <- numeric(prod(n))

  if(type == "1a" || type == 1L)
  {
    # one whole block
    index <- sapply(seq(1, ln[1] * n[2], n[1]), function(x) x:(x + ln[1] - 1))
  } else if(type == "1b")
  {
    # shift lies in two blocks
    index <- sapply(seq(1, ln[1] * n[2], n[1]),
                    function(x) (x + floor(ln[1] / 2)):(x + floor(1.5 * ln[1]) - 1))
  } else if(type == "1c")
  {
    # shift lies in four blocks
    index <- sapply(seq(floor(ln[1] / 2) * n[2], floor(1.5 * ln[1]) * n[2] - 1, n[1]),
                    function(x) (x + floor(ln[1] / 2) + 1):(x + floor(1.5 * ln[1])))
  } else if(type == 2L)
  {
    # half of the data
    index <- 1:(prod(n) / 2)
  } else if(type == 3L)
  {
    # steady increase from left to right
    return(matrix(rep(1 / sqrt(prod(n)) * (0:(n[2]-1)) / (n[2] - 1), each = n[1]), nrow = n[1]))
  } else if(type == 4L)
  {
    i1 <- floor(n / 4)# + as.integer((n) %% 4 > 0)
    i2 <- floor(n / 2)
    # i2 <- floor(n / 2) + as.integer((n) %% 4 > 0)
    
    index <- c(sapply(seq(i1[2] * n[1] + 1, i2[2] * n[1], n[1]), function(x) x:(x + i2[1] - 1)), 
               # rectangle
               sapply(seq(i1[1] + 1, i1[2] * n[1], n[1]), function(x) x:(x + (i2[1] - i1[1]) - 1)))
    # square
  } else if(type == 5L)
  {
    # field is divided into 10 "columns", every other column is shifted
    i <- floor(n[1] / 10)
    index <- c((n[2] * i + 1):(n[2] * 2 * i), (n[2] * 3 * i + 1):(n[2] * 4 * i), 
               (n[2] * 5 * i + 1):(n[2] * 6 * i), (n[2] * 7 * i + 1):(n[2] * 8 * i), 
               (n[2] * 9 * i + 1):(n[2] * 10 * i))
  } else if(type == 6L)
  {
    upper <- min(n)
    index <- unlist(sapply(1:upper, function(i) ((i - 1) * n[1] + 1):((i - 1) * n[1] + i)))
  } else if(type == 7L)
  {
    # "circle" including everything within distFun delta from middle

    if(missing(middle)) middle <- floor(n / 2)
    delta <- delta * distFun(rbind(c(1, 1), n))
    
    index <- do.call(rbind, sapply(1:n[1], function(x) 
    {
      res <- which(Vectorize(function(y) distFun(rbind(c(x, y), middle)))(1:n[2]) < delta)
      if(length(res) > 0) return(data.frame(x, res))
      return(NULL)
    }))
    index <- (index[, 2] - 1) * n[1] + index[, 1]
  } else if(type == 8)
  {
    m <- floor(n[2] / 2)
    index <- (n[1] * m + 1):(n[1] * (m + 1))
  }

  # alt[index] <- H / prod(n)^(0.43)
  return(as.integer(index))
}

test_that("autocov has the correct format", {
  X <- genField(c(20, 30))
  Sigma <- autocov(X, b = c(4, 4))
  Sigma1 <- autocov(X, b = 4, direction = 1)
  Sigma2 <- autocov(X, b = 4, direction = 2)
  
  expect_equal(dim(Sigma), c(600, 600))
  expect_equal(dim(Sigma1), c(30, 30))
  expect_equal(dim(Sigma2), c(20, 20))
  
  expect_equal(Sigma[1, 6], 0)
  expect_equal(Sigma[20, 1], 0)
  expect_true(Sigma[21, 1] != 0)
})

test_that("block_stat produces the correct format", {
  X <- genField(c(50, 50))
  
  y1 <- block_stat(X, 0.6, "gmd")
  y2 <- block_stat(X, 0.6, "var")
  y3 <- block_stat(X, 0.6, "jb")
  y4 <- block_stat(X, 0.6, "ks")
  y5 <- block_stat(X, 0.6, "grubbs")
  y6 <- block_stat(X, 0.6, "ANOVA")
  
  expect_equal(length(y1), 1)
  expect_equal(length(y2), 1)
  expect_equal(length(y3), 1)
  expect_equal(length(y4), 1)
  expect_equal(length(y5), 1)
  expect_equal(length(y6), 1)
  
  expect_equal(attributes(y5), list(n = 25, blocksize = c(10, 10)))
  expect_equal(attributes(y6), list(k = 25, N = 50^2, blocksize = c(10, 10)))
})

test_that("block_stat and block_pValue fit together", {
  tn <- replicate(1000, 
  {
    X <- genField(c(50, 50))
    block_stat(X, 0.6, "gmd")
  })
  expect_lt(mean(block_pValue(tn, "gmd") < 0.05), 0.1)
  
  tn <- replicate(1000, 
  {
    X <- genField(c(50, 50))
    block_stat(X, 0.6, "var")
  })
  expect_lt(mean(block_pValue(tn, "var") < 0.05), 0.1)
  
})

test_that("changeRegion returns the correct format", {
  expect_equal(class(changeRegion(c(50, 50), type = "1a")), "integer")
  expect_equal(class(changeRegion(c(50, 50), type = "1b")), "integer")
  expect_equal(class(changeRegion(c(50, 50), type = "1c")), "integer")
  expect_equal(class(changeRegion(c(50, 50), type = 2)), "integer")
  expect_equal(class(changeRegion(c(50, 50), type = 4)), "integer")
  expect_equal(class(changeRegion(c(50, 50), type = 5)), "integer")
  
  expect_equal(class(changeRegion(c(50, 50), type = 3)), c("matrix", "array"))
  expect_length(changeRegion(c(50, 50), type = 3), 50^2)
})

test_that("genField returns the correct format", {
  X <- genField(c(50, 40))
  
  expect_equal(dim(X), c(50, 40))
  expect_equal(class(X), "RandomField")
  
  x <- genField(50)
  
  expect_equal(length(x), 50)
  expect_equal(class(x), "RandomField")
})

#' Block test p-value
#'
#' Returns the p-value of a test statistic according to the block test for structural changes.
#' 
#' @param tn test statistic
#' @param fun Character string; one of "gmd" (default), "var", "jb", "grubbs", "ANOVA", "ad", "sw" (see \code{\link{block_stat}} for details).
#' 
#' @returns A numeric value between 0 and 1.
#' 
#' @export
#' @importFrom nortest ad.test
block_pValue <- function(tn, fun = "gmd")
{
  if(fun == "gmd" | fun == "var" | fun == "varWithoutCentering") return(1 - pnorm(tn))
  else if(fun == "jb") return(1 - pchisq(tn, 2))
  else if(fun == "grubbs") ## Grubbs: no real p-value
    return(as.numeric(tn < crit.grubbs(attr(tn, "n"), 0.05))) ### 0.05 ????
  else if(fun == "ANOVA") return(1 - pf(tn, attr(tn, "k") - 1, attr(tn, "N") - attr(tn, "k")))
  else if(fun == "ad") return(nortest::ad.test(tn)$p.value) 
  else if(fun == "sw") return(shapiro.test(tn)$p.value)
  
  stop("fun unknown")
}


#' Block test for structural changes
#' 
#' A test to detect whether an underlying time series or random field is stationary (hypothesis)
#' or if there is a location shift present in a region (alternative).
#'
#' @param x times series or random field to be tested. Either a numeric vector or a numeric matrix.
#' @param s parameter for the size of the blocks, 0.5 < s < 1, block length \eqn{l_n = [n^s]}. Default is \code{\link{sOpt}}\code{(n, 0.6)}.
#' @param fun Character string; one of "gmd" (default), "var", "jb", "grubbs", "ANOVA", "ad", "sw" (see \code{\link{block_stat}} for details).
#' @param varEstim variance estimator or variance estimation of the whole field or times series.
#'                 Either a function to estimate the variance with, or a numeric value.
#'
#' @return A list of the class "htest" containing the following components:
#'  \item{statistic}{value of the test statistic (numeric).}
#'  \item{p.value}{p-value (numeric).}
#'  \item{alternative}{alternative hypothesis (character string).}
#'  \item{method}{name of the performed test (character string).}
#'  \item{data.name}{name of the data (character string).}
#'
#' @seealso [block_stat], [block_pValue]
#' 
#' @references \enc{Görz}{Goerz}, S. and Fried, R. (2025). "Detecting changes in the mean of spatial random fields on a regular grid", \emph{arXiv preprint} \href{https://arxiv.org/abs/2512.11599}{arXiv:2512.11599}
#' @references Schmidt, S. K. (2024). Detecting changes in the trend function of heteroscedastic time series. Bernoulli, 30(4), 2598-2622.
#'
#' @examples
#' # time series with a shift 
#' x <- arima.sim(model = list(ar = 0.5), n = 100)
#' x[1:50] <- x[1:50] + 1
#' block_test(x, sOpt(100, 0.6))
#' 
#' # field without a shift and ordinary variance
#' X <- genField(c(50, 50))
#' block_test(X, sOpt(50, 0.6), "var")
#' 
#' # field with a shift and ordinary variance
#' X <- genField(c(50, 50), type = 2)
#' block_test(X, sOpt(50, 0.6), "var")
#' 
#' #' # GMD test statistic, scaling variance estimated by the mad
#' block_stat(X, 0.6, fun = "var", varEstim = mad)
#' 
#' @export
block_test <- function(x, s, fun = "gmd", varEstim = var)
{
  stat <- block_stat(x, s, fun, varEstim)
  names(stat) <- "S"
  pval <- block_pValue(stat, fun)
  
  Dataname <- deparse(substitute(x))
  
  erg <- list(alternative = "two-sided", method = paste(fun, "block test for structural changes"), 
              data.name = Dataname, statistic = stat, blocksize = attr(stat, "blocksize"),
              p.value = pval#, lrv = list(method = method, param = attr(stat, "param"), value = attr(stat, "sigma")), 
              )
  
  class(erg) <- "htest"
  return(erg)
}

test_that("block_test produces the correct output",
{
  x <- rnorm(c(50, 50))
  y <- block_test(x, s = 0.6, fun = "gmd")
  
  expect_true(is.list(y))
  expect_equal(class(y), "htest")
  
  expect_true(all(c("alternative", "method", "data.name",
                    "statistic", "p.value") %in% names(y)))
})

############# HELPERS ##########################################################

# #' Object dimensions
# #'
# #' Function to return the dimensions of both a vector and other objects
# #'     (like matrices and data.frames)
# #'
# #'
# dim2 <- function(x)
# {
#   if(is.vector(x)) return(c(length(x), 0))
#   return(dim(x))
# }

# RsSizes <- function(n, lower = 0.5, upper = 1, step = 0.1)
# {
#   s <- seq(lower, upper, step)
#   ln <- round(n^s)
#   bn <- floor(n / ln)
# 
#   data.frame(n = n, s = s, ln = ln, bn = bn, "ln*bn" = ln * bn, diff = n - (ln * bn))
# }
# 
# RsOpt <- function(n, s = 0.6)
# {
#   sapply(n, function(n)
#   {
#     res <- RsSizes(n, 0.5, 1, 0.001)
#     res <- res[res$diff == 0, ]
#     res$s[which.min(abs(res$s - s))]
#   })
# }

mu <- function(x, ln, fun = "mean")
{
  if(!is.function(fun)) fun <- eval(parse(text = fun))
  if(is.vector(x)) x <- matrix(x)
  if(is.na(ln[2])) ln[2] <- 1
  n <- dim(x)
  
  res <- sapply(seq(ln[1] + 1, n[1] + 1, ln[1]), function(i)
  {
    sapply(seq(ln[2] + 1, n[2] + 1, ln[2]), function(j)
    {
      fun(x[(i - ln[1]):(i - 1), (j - ln[2]):(j - 1)])
    })
  })
  return(res)
}

#' Skewness and kurtosis
#' 
#' Compute the skewness and the kurtosis of the data vector \code{x}.
#' 
#' @param x numeric vector.
#' 
#' @returns A numeric value.
#' 
#' @examples 
#' skewness(rnorm(100))
#' skewness(rexp(100, 2))
#' 
#' kurtosis(rnorm(100))
#' kurtosis(rt(100, 5))
#' @rdname higherMoments
#'@export
skewness <- function(x)
{
  n <- length(x)
  sigma <- var(x) * (n - 1) / n
  return(mean((x - mean(x))^3) / sigma^(3/2))
}

#' @rdname higherMoments
#'
#'@export
kurtosis <- function(x)
{
  n <- length(x)
  sigma <- var(x) * (n - 1) / n
  return(mean((x - mean(x))^4) / sigma^2)
}

#' Grubbs outlier test
#' 
#' Computes the test statistic and the critical value of the outlier test according to Grubbs.
#' 
#' @param x numeric vector.
#' @param varEstim Variance estimation or estimation function. Either a numeric value or a function taking one argument.
#' 
#' @return numeric value.
#' 
#' @examples
#' x <- rnorm(100)
#' grubbs(x) > crit.grubbs(100, 0.05)
#' 
#' # add outlier
#' x[1] <- x[1] + 100
#' grubbs(x) > crit.grubbs(100, 0.05)
#' 
#' 
#' @rdname grubbs
#' @export
grubbs <- function(x, varEstim = var)
{
  if(is.function(varEstim)) varEstim <- varEstim(x)
  return(max(abs(x - mean(x))) / sqrt(varEstim))
}


#' Critical value of the Grubbs test
#' 
#' @param n sample size; positive numeric value.
#' @param alpha significance level; between 0 and 1.
#' 
#' @rdname grubbs
#'@export
crit.grubbs <- function(n, alpha = 0.05)
{
  (n - 1) / sqrt(n) * sqrt(qt(alpha / 2 / n, n-2)^2 /
                             (n - 2 + qt(alpha / 2 / n, n-2)^2))
}

test_that("skewness and kurtosis return the correct values", {
  expect_equal(skewness(rnorm(100000)), 0, tolerance = 0.05)
  expect_equal(kurtosis(rnorm(100000)), 3, tolerance = 0.05)
})

test_that("Grubbs test and corresponding critical values are correctly computed", {
  y <- replicate(10000, {x <- rnorm(100); grubbs(x)})
  expect_equal(mean(y > crit.grubbs(100, 0.05)), 0.05, tolerance = 0.05)
})

#' Plot random field
#'
#' @param x \code{RandomField} object.
#' @param main title to the plot.
#' @param colors vector of colors over which a gradient is created.
#' @param name title of the legend, character string.
#' @param alpha opacity of the raster tiles. Numeric value between 0 and 1.
#' @param p additional ggplot2 component added BEFORE this function's components.
#' @param ... other parameters to be passed through to plotting functions.
#' 
#' @examples
#' plot(genField(c(50, 50)))
#' plot(genField(c(50, 50), type = 2))
#' 
#' @return a ggplot2 object. Function is mostly called for its side effect.
#'
#' @exportS3Method
#' @importFrom ggplot2 ggplot aes geom_tile scale_fill_gradientn ggtitle geom_raster guides guide_colorbar
plot.RandomField <- function(x, main = "", colors, name = "value", alpha = 1, p = NULL, ...)
{
  n <- dim(x)
  
  if(is.null(n) || (n[1] == 1 | n[2] == 1))
  {
    plot.ts(x)
    return(invisible(NULL))
  }
  
  if(missing(colors) || any(is.na(colors)))
    colors <- c(rgb(0.95, 1, 0), rgb(1, 1, 0), rgb(1, 0.8, 0), "orange", 
                "red", "darkred", "black")
  
  xvar <- as.numeric(colnames(x))
  if(length(xvar) == 0) xvar <- 1:n[2]
  yvar <- as.numeric(rownames(x)) 
  if(length(yvar) == 0) yvar <- 1:n[1]
  
  ### binding parameters for automatic check reasons
  y <- NA
  z <- NA
  ###

  data <- cbind(expand.grid(y = yvar, x = xvar), z = as.vector(x))
  p <- ggplot(data, aes(x = x, y = y, fill = z)) +
    p +
    geom_raster(alpha = alpha) +
    scale_fill_gradientn(colors = colors, name = name, ...) +
    ggtitle(main) +
    guides(fill = guide_colorbar(order = 1))

  return(p)
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' Gini's mean difference
#' 
#' Calculates Gini's mean difference of a given vector \code{x}.
#' 
#' @param x numeric vector.
#' 
#' @return A numeric value.
#' 
#' @examples
#' x <- rnorm(100)
#' GMD(x)
#' 
#' @export
GMD <- function(x) {
    .Call('_SChangeBlock_GMD', PACKAGE = 'SChangeBlock', x)
}

#' Mu
#' 
#' This function returns a vector of the block means for a given random field X.
#' 
#' @param x Numeric vector or matrix.
#' @param group strictly positive integer vector or matrix indicating the group (or block) of the corresponding observation in X.
#'        Overwrites `l` if specified.
#' @param l block length. Integer vector of length 1 or 2, depending on the number of dimensions of X, with strictly positive entries.
#' 
#' @return A numeric vector of length \code{floor(n[1] / l[1]) * floor(n[2] / l[2])}, \code{n = dim(x)}.
#' 
#' @examples 
#' X <- genField(c(50, 100), H = 100, type = 2)
#' M <- Mu(X, l = c(10, 20))
#' 
#' plot(X)
#' image(matrix(M, ncol = 5))
#' 
#' @export
Mu <- function(x, group = NULL, l = NULL) {
    .Call('_SChangeBlock_Mu', PACKAGE = 'SChangeBlock', x, group, l)
}

gamma <- function(X, h1, h2) {
    .Call('_SChangeBlock_gamma', PACKAGE = 'SChangeBlock', X, h1, h2)
}

gammaDiff <- function(X, h1, h2) {
    .Call('_SChangeBlock_gammaDiff', PACKAGE = 'SChangeBlock', X, h1, h2)
}

#' Autocovariance matrix
#' 
#' Estimates the autocovariance matrix for a given data matrix X. Via the parameter \code{direction}, it is possible to estimate only 
#' row- or columnwise autocovariance matrices, which is useful if the autocovariance function is separable.
#' 
#' @param X numeric matrix,
#' @param b numeric vector containing two integer values: the bandwidths for the row- resp. column-wise estimation.
#'        (Up to which lag should the autocovariances be estimated?) If \code{direction} > 0: only one integer must be supplied. 
#'        Bandwidths must be smaller than the dimensions of X.
#' @param M numeric vector containing two integer values, only needed for \code{type = 1}, see Details.
#' @param direction 0: all directions, 1: only row-wise autocovariances, 2: only column-wise autocovariances.
#' @param type 0: ordinary autocovariance estimation, 1: difference-based autocovariance estimation. See Details.
#' 
#' @returns A numeric matrix of size \eqn{N \times N}. If \code{direction = 0} then \eqn{N =} \code{prod(dim(X))}. 
#'          If \code{direction = 1} then \eqn{N =} \code{ncol(X)}, if \code{direction = 2} then \eqn{N =} \code{nrow(X)}.
#'          
#' @details In this function, the autocovariance matrix of \code{X} is interpreted as the autocovariance matrix of 
#'          \code{x = as.vector(X)}, i.e. where \code{X} is ordered into a vector column-wise. 
#'          If \code{type = 0}, the autocovariance to lags \eqn{h_1, h_2} is estimated using the regular estimator
#'          \deqn{\hat{\gamma}_{\text{reg}}(h_1, h_2) = 
#'           \frac{1}{(n-h_1)(m-h_2)}\sum_{i = 1}^{n-h_1} \sum_{j = 1}^{m-h_2} (Y_{i, j} - \bar{Y})(Y_{i+h_1, j+h_2} - \bar{Y}).}
#'          If \code{type = 1}, the autocovariance to lags \eqn{h_1, h_2} is estimated by a difference-based version, inspired by 
#'          the estimator of Tecuapetla-Gómez and Munk (2017) for time series:
#'          \deqn{\hat{\gamma}_{\text{diff}}(h_1, h_2) = \hat{\sigma}_{\text{diff}}^2 -
#'           \frac{1}{2(n - h_1)(m - h_2)}\sum_{i = 1}^{n-h_1} \sum_{j = 1}^{m-h_2} (Y_{i, j} - Y_{i + h_1, j + h_2})^2}
#'          with
#'          \deqn{\hat{\sigma}_{\text{diff}}^2 = \frac{1}{4} \left(\frac{1}{n(m - M_2)}\sum_{i = 1}^n \sum_{j = 1}^{m - M_2} 
#'          (Y_{i, j} - Y_{i, j+M_2})^2 + \frac{1}{(n - M_1)m}\sum_{i = 1}^{n-M_1} \sum_{j = 1}^{m} (Y_{i, j} - Y_{i+M_1, j})^2 \right),}
#'          where \eqn{M_1 =} \code{M[1]}, \eqn{M_2 = } \code{M[2]}.
#' 
#' @examples
#' X <- genField(c(20, 20))
#' autocov(X, c(4, 4))[1:10, 1:100]
#' 
#' # if separable:
#' Sigma1 <- autocov(X, 4, direction = 1)
#' Sigma2 <- autocov(X, 4, direction = 2)
#' kronecker(Sigma1, Sigma2)[1:10, 1:100]
#' 
#' @references
#' Tecuapetla‐Gómez, I., & Munk, A. (2017). Autocovariance estimation in regression with a discontinuous signal and m‐dependent errors: A difference‐based approach. Scandinavian Journal of Statistics, 44(2), 346-368.
#' 
#' @export
autocov <- function(X, b, M = as.integer( c(1, 1)), direction = 0L, type = 0L) {
    .Call('_SChangeBlock_autocov', PACKAGE = 'SChangeBlock', X, b, M, direction, type)
}

#' Dependency Matrix Theta
#' 
#' This function generates a symmetric dependency matrix \eqn{\Theta} of a 
#' specific type of spatial MA(q) model.
#' 
#' @param q model order (integer).
#' @param param parameter (numeric value between -1 and 1).
#' @param structure Character string, either "MA" or "AR" indicating the structure of the dependency matrix. Details below.
#' 
#' @return A matrix of size (2q + 1) x (2q + 1).
#' 
#' @details Symmetric spatial MA(q) model (or an approximation to a spatial AR(1) model) for 2-dim. random fields:
#'          \deqn{Y_{ij} = \sum_{k = -q}^q \sum_{l = -q}^q \theta_{kl} \varepsilon_{i+k, j+l}.}
#'          \eqn{(\theta_{kl}) = \Theta}. \cr \cr
#'          For "MA": \deqn{\theta_{kl} = \code{param}^{|k - q - 1| + |l - q - 1|}.}
#'          For "AR": \deqn{\theta_{kl} = \tilde{\theta}_{kl} / \sqrt{\sum_{|k| \leq q} \sum_{|l| \leq q} \tilde{\theta}^2_{kl}} 
#'          \quad \text{ with } \quad \tilde{\theta}_{kl} = \code{param}^{\sqrt{k^2 + l^2}}.}
#'          
#' @examples
#' genTheta(1, 0.2, "MA")
#' 
#' genTheta(40, 0.2, "AR")
#' 
#' @export
genTheta <- function(q, param, structure = "MA") {
    .Call('_SChangeBlock_genTheta', PACKAGE = 'SChangeBlock', q, param, structure)
}

dependency <- function(E, Theta_ = NULL, q_ = NULL, param_ = NULL) {
    .Call('_SChangeBlock_dependency', PACKAGE = 'SChangeBlock', E, Theta_, q_, param_)
}

#' @param n integer value.
#' @param lower,upper lower and upper search border, between 0 and 1.
#' @param step size of the step for the search, between 0 and 1.
#' 
#' 
#' @return \code{sSizes} returns a data frame containing
#' \item{n}{the given sample size}
#' \item{s}{the exponent in question}
#' \item{ln}{the resulting block length}
#' \item{bn}{the corresponding number of block}
#' \item{ln.bn}{block length times number of blocks}
#' \item{diff}{difference between the given sample size and the number of observations covered by the blocks}
#' 
#' @examples
#' sSizes(50)
#' sSizes(50, 0.6, 0.8, 0.01)
#' 
#' @rdname sOpt
#' @export
sSizes <- function(n, lower = 0.5, upper = 1, step = 0.1) {
    .Call('_SChangeBlock_sSizes', PACKAGE = 'SChangeBlock', n, lower, upper, step)
}

#' Optimal parameter s
#' 
#' Calculates the best parameter \eqn{\tilde{s}} for a given approximation s, such that \eqn{n \; % \; \[n^{s}\] = 0}.
#' 
#' @param n Sample size(s), numeric (vector).
#' @param s Desired exponent, \eqn{0.5 \leq s \leq 1}.
#' 
#' @return \code{sOpt} returns a numeric vector of the optimal exponent(s).
#' 
#' @examples 
#' sOpt(50, 0.6)
#' sOpt(100, 0.6)
#' 
#' @export
sOpt <- function(n, s = 0.6) {
    .Call('_SChangeBlock_sOpt', PACKAGE = 'SChangeBlock', n, s)
}

test_that("dependency matrix Phi is correct", {
  thetaAR <- genTheta(1, 0.4, structure = "AR")
  
  expect_true("matrix" %in% class(thetaAR))
  expect_equal(dim(thetaAR), c(3, 3))
  theta_test <- matrix(c(0.4^sqrt(2), 0.4, 0.4^sqrt(2), 
                         0.4, 1, 0.4, 
                         0.4^sqrt(2), 0.4, 0.4^sqrt(2)), ncol = 3)
  theta_test <- theta_test / sqrt(sum(theta_test^2))
  expect_equal(thetaAR, theta_test)
  
  thetaMA <- genTheta(1, 0.4, structure = "MA")
  theta_test <- matrix(c(0.4^2, 0.4, 0.4^2, 
                         0.4, 1, 0.4, 
                         0.4^2, 0.4, 0.4^2), ncol = 3)
  expect_equal(thetaMA, theta_test)
  
  phi <- genTheta(2, c(0.8, 0.6, 0.4, 0.2))
  expect_equal(dim(phi), c(5, 5))
  expect_equal(phi, matrix(c(0.2, 0.4, 0.6, 0.4, 0.2, 
                             0.4, 0.6, 0.8, 0.6, 0.4, 
                             0.6, 0.8, 1, 0.8, 0.6, 
                             0.4, 0.6, 0.8, 0.6, 0.4, 
                             0.2, 0.4, 0.6, 0.4, 0.2), ncol = 5))
})

test_that("dependency is correctly added to the data", 
{
  E <- matrix(rnorm(25), ncol = 5)
  theta <- genTheta(2, 0.4)
  X1 <- dependency(E, q_ = 2, param_ = 0.4)
  X2 <- dependency(E, Theta_ = theta)
  Xtest <- sum(E * theta)
  
  expect_equal(X1, X2)
  expect_true("matrix" %in% class(X1))
  expect_equal(dim(X1), c(1, 1))
  expect_equal(X1[1], Xtest)
  expect_equal(X2[1], Xtest)
})

#' Block test statistic 
#'
#' Computes test statistics of the block test on structural changes.
#'
#' @param x times series or random field to be tested. Either a numeric vector or a numeric matrix.
#' @param s parameter for the size of the blocks, 0.5 < s < 1, block length \eqn{l_n = [n^s]}. Default is \code{\link{sOpt}}\code{(n, 0.6)}.
#' @param fun Character string; one of "gmd" (default), "var", "jb", "grubbs", "ANOVA".
#' @param varEstim variance estimator or variance estimation of the whole field or times series.
#'                 Either a function to estimate the variance with, or a numeric value.
#'                 
#' @details
#' First, the time series or random field is divided into blocks and the means of 
#' the blocks are computed. Then the function \code{fun} is applied to the block means: 
#' * gmd: Gini's mean difference
#' * var: Ordinary variance estimator
#' * jb: Jarque-Bera test
#' * grubbs: Grubbs test for outliers
#' * ANOVA: simple ANOVA.
#' 
#' @returns A numeric value. 
#' For \code{fun} = \code{"grubbs"} it has the attribute \code{n} 
#' indicating the number of blocks, i.e. the number of observations used in the Grubbs test.
#' For \code{fun} = \code{"ANOVA"} it has the attributes \code{k} (number of blocks) and
#' \code{N} (total number of observations).
#' 
#' @seealso [block_pValue]
#' 
#' @examples
#' # time series with a shift 
#' x <- arima.sim(model = list(ar = 0.5), n = 100)
#' x[1:50] <- x[1:50] + 1
#' block_stat(x, sOpt(100, 0.6))
#' 
#' # field without shift and ordinary variance
#' X <- genField(c(50, 50))
#' block_stat(X, sOpt(50, 0.6), "var")
#' 
#' # field with a shift and ordinary variance
#' X <- genField(c(50, 50), type = 2)
#' block_stat(X, sOpt(50, 0.6), "var")
#' 
#' # GMD test statistic, scaling variance estimated by the mad
#' block_stat(X, 0.6, fun = "var", varEstim = mad)
#'
#' @importFrom methods is
#' @export
block_stat <- function(x, s, fun = "gmd", varEstim = var)
{
  if(is.vector(x) | is.ts(x)) n <- length(x) else n <- dim(x)
  
  if(missing(s)) s <- sOpt(n, 0.6)

  if(length(s) == 1 || length(s) == length(n))
  {
    # CAREFUL: bn is the product of all dimensions, but ln is still a vector since
    # it is needed for the ANOVA
    ln <- round(n^s)
    bn <- prod(floor(n / ln))
    m <- Mu(x, l = ln)
  } else if( ((is(x, "vector") && is(s, "vector")) && length(x) == length(s)) || 
             (length(dim(x)) == length(dim(s)) && all(dim(x) == dim(s))) )
  {
    if(length(grep("var", fun)) < 1) stop("Individual block sizes only possible for 'var'")
    ln <- table(s)
    bn <- length(ln)
    attributes(ln) <- NULL
    m <- Mu(x, group = s)
  } else
  {
    stop("no valid option for s")
  }
  
  if(is.function(varEstim))
  {
    sigma_x <- varEstim(as.vector(x))
    sigma_mu <- varEstim(m)
  } else
  {
    sigma_x <- varEstim
    ### ????
    sigma_mu <- varEstim / prod(ln)
  }
  
  
  if(fun == "gmd")
  {
    # Gini's mean difference
    res <- sqrt(bn) * (sqrt(prod(ln)) / sqrt(sigma_x) * GMD(m) - 2 / sqrt(pi)) /
      sqrt(4/3 + 8/pi * (sqrt(3) - 2))
  } else if(fun == "var")
  {
    # variance
    # res <- (var(m * sqrt(prod(ln))) * (bn - 1) / sqrt(bn)) / (sigma_x * sqrt(2)) - (bn - 1) / sqrt(2 * bn)
    kappa <- 2#mean((m * sqrt(prod(ln)))^4) - 1
    if(length(ln) == length(m)) nu <- ln * m^2 else nu <- prod(ln) * m^2
    res <- (1 / sigma_x * (sum(nu) - prod(n) * mean(m)^2) - bn + 1) / sqrt(kappa * bn)
  } else if(fun == "varWithoutCentering")
  {
    if(length(ln) == length(m)) nu <- ln * m^2 else nu <- prod(ln) * m^2
    res <- (sum(nu) - bn * sigma_x) / (sigma_x * sqrt(2 * bn))
  } else if(fun == "jb")
  {
    # Jarque-Bera
    s <- skewness(m)
    k <- kurtosis(m)
    
    res <- bn / 6 * (s^2 + (k - 3)^2 / 4)
  } else if(fun == "ks")
  {
    # KS statistics
    normvalues <- pnorm(sort(m), 0, sqrt(sigma_x / prod(ln)))
    res <- max(abs(c(normvalues - (1:bn) / bn, normvalues - (0:(bn-1)) / bn))) * sqrt(bn)
  } else if(fun == "grubbs")
  {
    # Grubbs test for outliers
    res <- grubbs(m, sigma_mu)
    attr(res, "n") <- bn
  } else if(fun == "ANOVA")
  {
    mu.bar <- mean(m)
    
    MST <- sigma_mu * prod(n)
    MSE <- sum(mu(x, ln, function(x) var(as.vector(x))))
    
    res <- MST / MSE
    #if(locfun == "median") res <- res * 2 / pi
    
    attr(res, "k") <- bn
    attr(res, "N") <- prod(n)
  } else
  {
    res <- m
  } 
  # else
  # {
  #   stop("Invalid argument for 'fun'")
  # }
  
  attr(res, "blocksize") <- ln
  
  return(res)
}

test_that("outout from Mu has the correct format", {
  X <- matrix(rnorm(36), ncol = 6)
  
  expect_equal(length(Mu(X, l = c(2, 2))), 9)
  expect_equal(length(Mu(X, l = c(3, 3))), 4)
  expect_equal(length(Mu(X, group = matrix(c(rep(1, 6), rep(2, 30)), ncol = 6))), 2)
  # expect_equal(length(Mu(X, c(3, 2), c(1, 1))), 12)
  
  x <- rnorm(6)
  expect_equal(length(Mu(x, l = 2)), 3)
})

test_that("Mu computes the correct block sums", {
  # vector:
  set.seed(1045)
  x <- rnorm(100)
  expect_equal(Mu(x, l = 50), c(mean(x[1:50]), mean(x[51:100])))
  
  # matrix:
  set.seed(1045)
  X <- matrix(rnorm(36), ncol = 6)
  
  expect_equal(Mu(X, l = c(2, 2)), c(0.6946552, -0.4640347, 0.08525641,
                                     -0.1936238, -0.1491637, -0.4745453,
                                     -0.4166834, -0.112317, -0.5687769), 
               tolerance = 1e-7)
  expect_equal(Mu(X, l = c(3, 3)), c(-0.04461205, -0.4336385, -0.327058, 0.09453829), 
               tolerance = 1e-7)
  
  # errors:
  expect_error(Mu(1:3, group = 1))
  expect_error(Mu(rnorm(10), l = 0))
  expect_error(Mu(X, l = c(3, 0)))
})

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{genTheta}
\alias{genTheta}
\title{Dependency Matrix Theta}
\usage{
genTheta(q, param, structure = "MA")
}
\arguments{
\item{q}{model order (integer).}

\item{param}{parameter (numeric value between -1 and 1).}

\item{structure}{Character string, either "MA" or "AR" indicating the structure of the dependency matrix. Details below.}
}
\value{
A matrix of size (2q + 1) x (2q + 1).
}
\description{
This function generates a symmetric dependency matrix \eqn{\Theta} of a
specific type of spatial MA(q) model.
}
\details{
Symmetric spatial MA(q) model (or an approximation to a spatial AR(1) model) for 2-dim. random fields:
\deqn{Y_{ij} = \sum_{k = -q}^q \sum_{l = -q}^q \theta_{kl} \varepsilon_{i+k, j+l}.}
\eqn{(\theta_{kl}) = \Theta}. \cr \cr
For "MA": \deqn{\theta_{kl} = \code{param}^{|k - q - 1| + |l - q - 1|}.}
For "AR": \deqn{\theta_{kl} = \tilde{\theta}_{kl} / \sqrt{\sum_{|k| \leq q} \sum_{|l| \leq q} \tilde{\theta}^2_{kl}} 
         \quad \text{ with } \quad \tilde{\theta}_{kl} = \code{param}^{\sqrt{k^2 + l^2}}.}
}
\examples{
genTheta(1, 0.2, "MA")

genTheta(40, 0.2, "AR")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/block_pValue.R
\name{block_pValue}
\alias{block_pValue}
\title{Block test p-value}
\usage{
block_pValue(tn, fun = "gmd")
}
\arguments{
\item{tn}{test statistic}

\item{fun}{Character string; one of "gmd" (default), "var", "jb", "grubbs", "ANOVA", "ad", "sw" (see \code{\link{block_stat}} for details).}
}
\value{
A numeric value between 0 and 1.
}
\description{
Returns the p-value of a test statistic according to the block test for structural changes.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/decorr.R
\name{decorr}
\alias{decorr}
\title{De-correlation}
\usage{
decorr(X, lags, method = 1L, separable = FALSE, M = 1, type = 0)
}
\arguments{
\item{X}{Random Field, numeric matrix, or time series}

\item{lags}{numeric vector containing two integer values: the bandwidths for the row- reps. column-wise autocovariance estimation.
(Up to which lag should the autocovariances be estimated?) Lags must be smaller than the dimensions of X.}

\item{method}{1L: square root of the matrix via \code{\link[robcp:modifChol]{robcp::modifChol()}}, inversion via \code{\link[=solve]{solve()}} \cr
2L: square root and inversion via singular value decomposition \cr
3L: square root via \code{\link[expm:sqrtm]{expm::sqrtm()}}, inversion via \code{\link[=solve]{solve()}} \cr
Otherwise, a custom function for the square root and the inversion can be supplied via \code{method}. This function must take exactly one argument, which is the matrix \code{X}.}

\item{separable}{if the autocovariance function is (assumed to be) separable in the two directions of X,
those two autocovariances can be estimated separately and then combined (after square root and inversion) as a Kronecker product.}

\item{M}{numeric vector containing two integer values, only needed for \code{type = 1}, see \code{\link[=autocov]{autocov()}}.}

\item{type}{0: ordinary autocovariance estimation, 1: difference-based autocovariance estimation. See \code{\link[=autocov]{autocov()}}.}
}
\value{
De-correlated random field or time series; same data type and size as input \code{X}.
}
\description{
De-correlates a random field or time series, so that the resulting values can be treated as independent.
}
\details{
The contents of \code{X} are ordered into a vector \eqn{x} column-wise. The autocovariance matrix \eqn{\Sigma} of \eqn{x} is estimated by \code{\link[=autocov]{autocov()}}.
\eqn{\Sigma} is taken the square root of and being inverted using the functions specified in \code{method}. Then
\deqn{y = \Sigma^{-\frac{1}{2}} (x - \bar{x}).}
Then \eqn{y} is ordered back into a matrix \eqn{Y} with the same dimension as \eqn{X}.
}
\examples{
x <- arima.sim(list(ar = 0.4), 200)
y <- decorr(x, 3)
\donttest{
oldpar <- par(mfrow = c(2, 2))
acf(x)
pacf(x)
acf(y)
pacf(y)
par(oldpar) }

X <- genField(c(20, 20), Theta = genTheta(1, 0.4))
Y <- decorr(X, c(2, 2))

\donttest{
# custom function:
require(robcp)
Y2 <- decorr(X, c(2, 2), method = function(x) modifChol(solve(x))) }


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/genField.R
\name{genField}
\alias{genField}
\title{Generate random fields}
\usage{
genField(
  n,
  distr = rnorm,
  type = 0L,
  H = 100,
  Theta = NULL,
  q = NULL,
  param = NULL,
  ...
)
}
\arguments{
\item{n}{dimensions of the requested random field. Numeric vector.}

\item{distr}{function specifying the error distribution.}

\item{type}{change region type (integer or character). See \code{\link{changeRegion}}.}

\item{H}{height of the location shift (numeric value).}

\item{Theta}{matrix specifying the dependency between observations of a 2-dim random field.
Has to be a 2-dim. matrix where there is an odd number of entries in both dimensions.
Explanations are given in \link{genTheta}.}

\item{q}{dependency parameter for the model order of the MA field (integer > 0).}

\item{param}{dependency parameters for the model parameters of the MA field (numeric vector.)
Both \code{q} and \code{param} are ignored if \code{Theta} is supplied.
A dependency matrix is generated using \code{\link{genTheta}}.}

\item{...}{additional arguments for the generation of \code{\link{changeRegion}}.}
}
\value{
The function returns a matrix of dimension \code{n} that has the \code{\link{class}} "RandomField".
}
\description{
Generate random fields on a regular grid. Dependency can be modeled to some extend.
}
\details{
Set \code{dim(X)} \eqn{= (n, m)} and \eqn{N = nm}. The dependent random field follows the equation:
\deqn{Y_{ij} = \sum_{k = -q}^q \sum_{l = -q}^q \theta_{kl} \varepsilon_{i+k, j+l} + \frac{H}{\sqrt{N}} \mathbb{1}_{\{(i, j) \in \mathbb{A}\}}}
where \eqn{\mathbb{A} = \{(i, j) \in \{1, ..., n\} \times \{1, ..., m\} | \text{ change region at } (i, j)\}}.
Only for \code{type = 3L} the term \eqn{\mathbb{1}_{\{(i, j) \in \mathbb{A}\}}} is replaced by \eqn{\frac{H}{\sqrt{N}} \frac{j}{m - 1}}.
}
\examples{
genField(c(50, 50))
genField(c(50, 50), type = 3, Theta = genTheta(1, 0.2))

}
\seealso{
\link{changeRegion}, \link{genTheta}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{skewness}
\alias{skewness}
\alias{kurtosis}
\title{Skewness and kurtosis}
\usage{
skewness(x)

kurtosis(x)
}
\arguments{
\item{x}{numeric vector.}
}
\value{
A numeric value.
}
\description{
Compute the skewness and the kurtosis of the data vector \code{x}.
}
\examples{
skewness(rnorm(100))
skewness(rexp(100, 2))

kurtosis(rnorm(100))
kurtosis(rt(100, 5))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/block_pValue.R
\name{block_test}
\alias{block_test}
\title{Block test for structural changes}
\usage{
block_test(x, s, fun = "gmd", varEstim = var)
}
\arguments{
\item{x}{times series or random field to be tested. Either a numeric vector or a numeric matrix.}

\item{s}{parameter for the size of the blocks, 0.5 < s < 1, block length \eqn{l_n = [n^s]}. Default is \code{\link{sOpt}}\code{(n, 0.6)}.}

\item{fun}{Character string; one of "gmd" (default), "var", "jb", "grubbs", "ANOVA", "ad", "sw" (see \code{\link{block_stat}} for details).}

\item{varEstim}{variance estimator or variance estimation of the whole field or times series.
Either a function to estimate the variance with, or a numeric value.}
}
\value{
A list of the class "htest" containing the following components:
\item{statistic}{value of the test statistic (numeric).}
\item{p.value}{p-value (numeric).}
\item{alternative}{alternative hypothesis (character string).}
\item{method}{name of the performed test (character string).}
\item{data.name}{name of the data (character string).}
}
\description{
A test to detect whether an underlying time series or random field is stationary (hypothesis)
or if there is a location shift present in a region (alternative).
}
\examples{
# time series with a shift 
x <- arima.sim(model = list(ar = 0.5), n = 100)
x[1:50] <- x[1:50] + 1
block_test(x, sOpt(100, 0.6))

# field without a shift and ordinary variance
X <- genField(c(50, 50))
block_test(X, sOpt(50, 0.6), "var")

# field with a shift and ordinary variance
X <- genField(c(50, 50), type = 2)
block_test(X, sOpt(50, 0.6), "var")

#' # GMD test statistic, scaling variance estimated by the mad
block_stat(X, 0.6, fun = "var", varEstim = mad)

}
\references{
\enc{Görz}{Goerz}, S. and Fried, R. (2025). "Detecting changes in the mean of spatial random fields on a regular grid", \emph{arXiv preprint} \href{https://arxiv.org/abs/2512.11599}{arXiv:2512.11599}

Schmidt, S. K. (2024). Detecting changes in the trend function of heteroscedastic time series. Bernoulli, 30(4), 2598-2622.
}
\seealso{
\link{block_stat}, \link{block_pValue}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{grubbs}
\alias{grubbs}
\alias{crit.grubbs}
\title{Grubbs outlier test}
\usage{
grubbs(x, varEstim = var)

crit.grubbs(n, alpha = 0.05)
}
\arguments{
\item{x}{numeric vector.}

\item{varEstim}{Variance estimation or estimation function. Either a numeric value or a function taking one argument.}

\item{n}{sample size; positive numeric value.}

\item{alpha}{significance level; between 0 and 1.}
}
\value{
numeric value.
}
\description{
Computes the test statistic and the critical value of the outlier test according to Grubbs.
}
\examples{
x <- rnorm(100)
grubbs(x) > crit.grubbs(100, 0.05)

# add outlier
x[1] <- x[1] + 100
grubbs(x) > crit.grubbs(100, 0.05)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{Mu}
\alias{Mu}
\title{Mu}
\usage{
Mu(x, group = NULL, l = NULL)
}
\arguments{
\item{x}{Numeric vector or matrix.}

\item{group}{strictly positive integer vector or matrix indicating the group (or block) of the corresponding observation in X.
Overwrites \code{l} if specified.}

\item{l}{block length. Integer vector of length 1 or 2, depending on the number of dimensions of X, with strictly positive entries.}
}
\value{
A numeric vector of length \code{floor(n[1] / l[1]) * floor(n[2] / l[2])}, \code{n = dim(x)}.
}
\description{
This function returns a vector of the block means for a given random field X.
}
\examples{
X <- genField(c(50, 100), H = 100, type = 2)
M <- Mu(X, l = c(10, 20))

plot(X)
image(matrix(M, ncol = 5))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{sSizes}
\alias{sSizes}
\alias{sOpt}
\title{Optimal parameter s}
\usage{
sSizes(n, lower = 0.5, upper = 1, step = 0.1)

sOpt(n, s = 0.6)
}
\arguments{
\item{n}{Sample size(s), numeric (vector).}

\item{lower, upper}{lower and upper search border, between 0 and 1.}

\item{step}{size of the step for the search, between 0 and 1.}

\item{s}{Desired exponent, \eqn{0.5 \leq s \leq 1}.}
}
\value{
\code{sSizes} returns a data frame containing
\item{n}{the given sample size}
\item{s}{the exponent in question}
\item{ln}{the resulting block length}
\item{bn}{the corresponding number of block}
\item{ln.bn}{block length times number of blocks}
\item{diff}{difference between the given sample size and the number of observations covered by the blocks}

\code{sOpt} returns a numeric vector of the optimal exponent(s).
}
\description{
Calculates the best parameter \eqn{\tilde{s}} for a given approximation s, such that \eqn{n \; \% \; [n^{s}] = 0}.
}
\examples{
sSizes(50)
sSizes(50, 0.6, 0.8, 0.01)

sOpt(50, 0.6)
sOpt(100, 0.6)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/block_stat.R
\name{block_stat}
\alias{block_stat}
\title{Block test statistic}
\usage{
block_stat(x, s, fun = "gmd", varEstim = var)
}
\arguments{
\item{x}{times series or random field to be tested. Either a numeric vector or a numeric matrix.}

\item{s}{parameter for the size of the blocks, 0.5 < s < 1, block length \eqn{l_n = [n^s]}. Default is \code{\link{sOpt}}\code{(n, 0.6)}.}

\item{fun}{Character string; one of "gmd" (default), "var", "jb", "grubbs", "ANOVA".}

\item{varEstim}{variance estimator or variance estimation of the whole field or times series.
Either a function to estimate the variance with, or a numeric value.}
}
\value{
A numeric value.
For \code{fun} = \code{"grubbs"} it has the attribute \code{n}
indicating the number of blocks, i.e. the number of observations used in the Grubbs test.
For \code{fun} = \code{"ANOVA"} it has the attributes \code{k} (number of blocks) and
\code{N} (total number of observations).
}
\description{
Computes test statistics of the block test on structural changes.
}
\details{
First, the time series or random field is divided into blocks and the means of
the blocks are computed. Then the function \code{fun} is applied to the block means:
\itemize{
\item gmd: Gini's mean difference
\item var: Ordinary variance estimator
\item jb: Jarque-Bera test
\item grubbs: Grubbs test for outliers
\item ANOVA: simple ANOVA.
}
}
\examples{
# time series with a shift 
x <- arima.sim(model = list(ar = 0.5), n = 100)
x[1:50] <- x[1:50] + 1
block_stat(x, sOpt(100, 0.6))

# field without shift and ordinary variance
X <- genField(c(50, 50))
block_stat(X, sOpt(50, 0.6), "var")

# field with a shift and ordinary variance
X <- genField(c(50, 50), type = 2)
block_stat(X, sOpt(50, 0.6), "var")

# GMD test statistic, scaling variance estimated by the mad
block_stat(X, 0.6, fun = "var", varEstim = mad)

}
\seealso{
\link{block_pValue}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/genField.R
\name{changeRegion}
\alias{changeRegion}
\title{Change Region}
\usage{
changeRegion(n, s, type = 1L, middle, delta = 0.15, distFun = dist)
}
\arguments{
\item{n}{dimensions of the random field. Numeric vector.}

\item{s}{parameter for the size of the blocks, 0.5 < s < 1, block length \eqn{l_n = [n^s]}.}

\item{type}{change region type (integer or character). See "Details".}

\item{middle, delta, distFun}{parameters for type 4L. See "Details".}
}
\value{
\item{Types 1a, 1b, 1c, 2, 4, 5}{A vector of indices.}
\item{Type 3}{A numeric (n x n) matrix containing the heights of the shifts at the corresponding locations.}
}
\description{
Generates the indices for different types of change regions.
}
\details{
Change region types:
\itemize{
\item 1L or "1a": exactly one block is shifted
\item "1b": size of the shift region is one block, but the shift region lies in two blocks
\item "1c": size of the shift region is one block, but the shift region lies in four blocks
\item 2L: exactly half of the data is shifted
\item 3L: there is a steady increase from left to right
\item 4L: shift is at \eqn{\{X_{ij} | (i \leq \frac{n}{2} \wedge \frac{m}{4} < j \leq \frac{m}{2}) \vee (\frac{n}{4} < i \leq \frac{n}{2} \wedge j \leq \frac{m}{4})\}}
\item 5L: for demonstration purposes: the field is divided into 10 "columns". Every other column is shifted
\item 6L: shift is at \eqn{\{X_{ij} | i \leq j \leq m\}} (upper triangle)
\item 7L: a "circle" including everything within \code{distFun} \code{delta} from \code{middle} is shifted
}
}
\examples{
changeRegion(c(50, 50), 0.6, "1a")
changeRegion(c(50, 50), type = 2)
changeRegion(c(50, 50), type = 3L)
changeRegion(c(50, 50), type = 7L, middle = c(10, 10))

}
\seealso{
\link{genField}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{autocov}
\alias{autocov}
\title{Autocovariance matrix}
\usage{
autocov(X, b, M = as.integer(c(1, 1)), direction = 0L, type = 0L)
}
\arguments{
\item{X}{numeric matrix,}

\item{b}{numeric vector containing two integer values: the bandwidths for the row- resp. column-wise estimation.
(Up to which lag should the autocovariances be estimated?) If \code{direction} > 0: only one integer must be supplied.
Bandwidths must be smaller than the dimensions of X.}

\item{M}{numeric vector containing two integer values, only needed for \code{type = 1}, see Details.}

\item{direction}{0: all directions, 1: only row-wise autocovariances, 2: only column-wise autocovariances.}

\item{type}{0: ordinary autocovariance estimation, 1: difference-based autocovariance estimation. See Details.}
}
\value{
A numeric matrix of size \eqn{N \times N}. If \code{direction = 0} then \eqn{N =} \code{prod(dim(X))}.
If \code{direction = 1} then \eqn{N =} \code{ncol(X)}, if \code{direction = 2} then \eqn{N =} \code{nrow(X)}.
}
\description{
Estimates the autocovariance matrix for a given data matrix X. Via the parameter \code{direction}, it is possible to estimate only
row- or columnwise autocovariance matrices, which is useful if the autocovariance function is separable.
}
\details{
In this function, the autocovariance matrix of \code{X} is interpreted as the autocovariance matrix of
\code{x = as.vector(X)}, i.e. where \code{X} is ordered into a vector column-wise.
If \code{type = 0}, the autocovariance to lags \eqn{h_1, h_2} is estimated using the regular estimator
\deqn{\hat{\gamma}_{\text{reg}}(h_1, h_2) = 
          \frac{1}{(n-h_1)(m-h_2)}\sum_{i = 1}^{n-h_1} \sum_{j = 1}^{m-h_2} (Y_{i, j} - \bar{Y})(Y_{i+h_1, j+h_2} - \bar{Y}).}
If \code{type = 1}, the autocovariance to lags \eqn{h_1, h_2} is estimated by a difference-based version, inspired by
the estimator of Tecuapetla-Gómez and Munk (2017) for time series:
\deqn{\hat{\gamma}_{\text{diff}}(h_1, h_2) = \hat{\sigma}_{\text{diff}}^2 -
          \frac{1}{2(n - h_1)(m - h_2)}\sum_{i = 1}^{n-h_1} \sum_{j = 1}^{m-h_2} (Y_{i, j} - Y_{i + h_1, j + h_2})^2}
with
\deqn{\hat{\sigma}_{\text{diff}}^2 = \frac{1}{4} \left(\frac{1}{n(m - M_2)}\sum_{i = 1}^n \sum_{j = 1}^{m - M_2} 
         (Y_{i, j} - Y_{i, j+M_2})^2 + \frac{1}{(n - M_1)m}\sum_{i = 1}^{n-M_1} \sum_{j = 1}^{m} (Y_{i, j} - Y_{i+M_1, j})^2 \right),}
where \eqn{M_1 =} \code{M[1]}, \eqn{M_2 = } \code{M[2]}.
}
\examples{
X <- genField(c(20, 20))
autocov(X, c(4, 4))[1:10, 1:100]

# if separable:
Sigma1 <- autocov(X, 4, direction = 1)
Sigma2 <- autocov(X, 4, direction = 2)
kronecker(Sigma1, Sigma2)[1:10, 1:100]

}
\references{
Tecuapetla‐Gómez, I., & Munk, A. (2017). Autocovariance estimation in regression with a discontinuous signal and m‐dependent errors: A difference‐based approach. Scandinavian Journal of Statistics, 44(2), 346-368.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Graphics.R
\name{plot.RandomField}
\alias{plot.RandomField}
\title{Plot random field}
\usage{
\method{plot}{RandomField}(x, main = "", colors, name = "value", alpha = 1, p = NULL, ...)
}
\arguments{
\item{x}{\code{RandomField} object.}

\item{main}{title to the plot.}

\item{colors}{vector of colors over which a gradient is created.}

\item{name}{title of the legend, character string.}

\item{alpha}{opacity of the raster tiles. Numeric value between 0 and 1.}

\item{p}{additional ggplot2 component added BEFORE this function's components.}

\item{...}{other parameters to be passed through to plotting functions.}
}
\value{
a ggplot2 object. Function is mostly called for its side effect.
}
\description{
Plot random field
}
\examples{
plot(genField(c(50, 50)))
plot(genField(c(50, 50), type = 2))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rmix.R
\name{rmix}
\alias{rmix}
\title{Mixture distribution}
\usage{
rmix(n, q = 0.01, h = 10, sigma = 1)
}
\arguments{
\item{n}{sample size, numeric.}

\item{q}{probability that observation is drawn from the contamination distribution, numeric.}

\item{h}{mean of the contamination distribution, numeric.}

\item{sigma}{standard deviation of the contamination distribution, numeric.}
}
\value{
Numeric vector of length n containing the random sample.
}
\description{
Generates a random sample from a mixture normal distribution.
}
\details{
The resulting sample is drawn from the distribution
\deqn{(1 - q)\mathcal{N}(0, 1)\; + \; q \mathcal{N}(h, \sigma^2).}
}
\examples{
# random sample with 0.01 chance of contamination distribution with mean 10
rmix(100)

# random sample with 0.01 chance of contamination distribution with standard deviation 10
# IMPORTANT: h needs to be set to 0!
rmix(100, h = 0, sigma = 1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/decorr.R
\name{bandwidth}
\alias{bandwidth}
\title{Bandwidth estimation}
\usage{
bandwidth(X, p1 = 0.3, p2 = 0.3, lag = 1)
}
\arguments{
\item{X}{numeric vector or matrix.}

\item{p1, p2}{exponents for sample size n resp. estimated dependency, between 0 and 1.}

\item{lag}{lag to which the autocorrelations are to be estimated. Integer > 0 but smaller than the length resp. number of rows and columns of X.}
}
\value{
A numeric vector containing one or two elements, depending on if a vector or matrix is supplied.
In case of a matrix: the first value is the bandwidth for the row-wise
and the second one for the column-wise estimation.
}
\description{
Calculate MSE-optimal bandwidths according to Andrews (1991).
}
\details{
Bandwidth \eqn{\boldsymbol{b}^{(n,m)} = (b_1^{(n)}, b_2^{(m)})} is estimated via
\deqn{b_i^{(k)} = \min\left(k-1, \max\left(1, k^{0.3} \left(\frac{2\rho_i}{1 - \rho_i^2}\right)^{0.3}\right)\right),}
where \eqn{\rho_1} and \eqn{\rho_2} are the mean row- resp. column-wise Spearman autocorrelations to lag 1.
}
\examples{
X1 <- genField(c(50, 50), Theta = genTheta(1, 0.4))
bandwidth(X1, 0.3, 0.3)

Theta <- matrix(c(0.08, 0.1, 0.08, 0.8, 1, 0.8, 0.08, 0.1, 0.08), ncol = 3)
X2 <- genField(c(50, 50), Theta = Theta)
bandwidth(X2, 1/3, 2/3)

}
\references{
Andrews, D. W. (1991). “Heteroskedasticity and autocorrelation consistent covariance
matrix estimation”. In: Econometrica: Journal of the Econometric Society, pp. 817–858.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{GMD}
\alias{GMD}
\title{Gini's mean difference}
\usage{
GMD(x)
}
\arguments{
\item{x}{numeric vector.}
}
\value{
A numeric value.
}
\description{
Calculates Gini's mean difference of a given vector \code{x}.
}
\examples{
x <- rnorm(100)
GMD(x)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SChangeBlock-package.R
\docType{package}
\name{SChangeBlock-package}
\alias{SChangeBlock}
\alias{SChangeBlock-package}
\title{SChangeBlock: Spatial Structural Change Detection by an Analysis of Variability Between Blocks of Observations}
\description{
Provides methods to detect structural changes in time series or random fields (spatial data). Focus is on the detection of abrupt changes or trends in independent data, but the package also provides a function to de-correlate data with dependence. The functions are based on the test suggested in Schmidt (2024) \url{doi:10.3150/23-BEJ1686} and the work in Görz and Fried (2025+).
}
\author{
\strong{Maintainer}: Sheila Goerz \email{sheila.goerz@tu-dortmund.de}

Other contributors:
\itemize{
  \item Roland Fried \email{msnat@statistik.tu-dortmund.de} [contributor, thesis advisor]
}

}
\keyword{internal}
